tic; t0=toc;  
%% Initial screen
clc;
fprintf('%s\n','%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');
fprintf('%s\n','%%% Uncertainty quantification by Particle Disparity	%%%');
fprintf('%s\n','%%%                                                     %%%');
fprintf('%s\n','%%% Dr. A. Sciacchitano, TU Delft, The Netherlands      %%%');
fprintf('%s\n','%%% July 2016                                           %%%');
fprintf('%s\n','%%%                                                     %%%');
fprintf('%s\n','%%% Sciacchitano A, Wieneke B and Scarano F (2013),     %%%'); 
fprintf('%s\n','%%% PIV uncertainty quantification by image matching,   %%%'); 
fprintf('%s\n','%%% Meas. Sci. Technol. 24 045302 (16pp)                %%%'); 
fprintf('%s\n','%%%                                                     %%%');
fprintf('%s\n','%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');
fprintf('%s\n',' ');


%% additional parameters
SaveFlag='yes';
fileroot='B';                      % root of the file
zerostr='00000';                     % string of zeros
disp_app='.dat';                    % format of the displacement fields
app_write='.dat';                    % format of the displacement fields
im_app='.tif';                      % format of the images
wsOdd=ceil(ws+0.01);

dt=1;
AcqMode='SingleFrame'; % 'SingleFrame' or DoubleFrame
Ncameras=1;
cam=1;
frame=1; % 1 or 2 (in the dat file)

% Processing Parameters
rsearch=1;  %search radius
nRmsLength=1; % kernel of the low pass filter (particle image diameter)
weight='sqrt'; %'peaks', 'I1I2', 'sqrt'


field_tot=(FileEnd-FileStart)/FileStep+1;     % total number of displacement fields

%% parallel processing
if isempty(gcp('nocreate'))==1;
    parpool;
end

%% create the folder fold_write (where the data are written) if it doesn't exist
if strcmp(SaveFlag,'yes')
    if exist(fold_write,'dir')~=7
        mkdir(fold_write);
    end
%     zip([fold_write '_INFO.zip'],{'*.m', 'PathFunctions\*.m'});
    write_info;
end
ind=0;

%% loop on the displacement fields
for field=FileStart:FileStep:FileEnd 
    ind=ind+1;
    fieldstr=num2str(field);
    field_p1str=num2str(field+dt);
    disp(['Field ' fieldstr]);
    
        
    %% read the two images of a pair
    t1=toc;
    fprintf('Read the images: ')
    
    [Xpx,Ypx,im]=ReadImagesTif(im_fold,fileroot,field,dt,zerostr,im_app);
    t2=toc;
    fprintf([num2str(t2-t1,'%4.1f') ' s.\n'])

    % sliding avg subtraction
    fprintf('Sliding average subtraction: ')
    im1Avg=slidingAvgDavis(im(:,:,1),wsOdd);
    im(:,:,1)=im(:,:,1)-im1Avg;
    clear im1Avg
    im2Avg=slidingAvgDavis(im(:,:,2),wsOdd);
    im(:,:,2)=im(:,:,2)-im2Avg; 
    clear im2Avg
    t3=toc;
    fprintf([num2str(t3-t2,'%4.1f') ' s.\n'])


    %% read the displacement field
    fprintf('Read the displacement field: ')
    FileRead = [fileroot zerostr(1:end-length(fieldstr)) fieldstr disp_app];
    [Xu_px,Yu_px,u,v,lenx,leny] = ReadDat([disp_fold FileRead]);
    grdx = Xu_px(1,:);
    grdy = Yu_px(:,1);
    % displacement in pixels (v [px] is positive upwards)
    displ_px=cat(3,u,v);
    t4=toc;
    fprintf([num2str(t4-t3,'%4.2f') ' s.\n'])

    %% image deformation (of the image pair) through the sinc interpolation
    fprintf('Image deformation: ')
    imdef=image_def(im,displ_px,Xu_px,Yu_px); %size (J,I,2);
    t5=toc;
    fprintf([num2str(t5-t4,'%4.1f') ' s.\n'])

    %% Gaussian Smoothing
    imdef1=squeeze(imdef(:,:,1));
    imdef2=squeeze(imdef(:,:,2));
    clear imdef
    imdef1=slidingAvgDavis(imdef1,nRmsLength);
    imdef2=slidingAvgDavis(imdef2,nRmsLength);

    %% disparity matrices computation
    fprintf('Disparity computation: ')
    t5=toc;
    [xdispl,ydispl,immult,peaks]=disparity_computation(imdef1,imdef2,rsearch,weight);
    t6=toc;
    fprintf([num2str(t6-t5,'%4.1f') ' s.\n'])

    %% Uncertainty computation
    fprintf('Uncertainty computation: ')   
    [extot,exbias,exrms,Npartu,Cu]=error_window4(xdispl,peaks,grdx,grdy,wsOdd,nRmsLength,window_weight,ROI); 
    [eytot,eybias,eyrms,Npartv,Cv]=error_window4(ydispl,peaks,grdx,grdy,wsOdd,nRmsLength,window_weight,ROI);
        
    %% total error
    t7=toc;
    fprintf([num2str(t7-t6,'%4.1f') ' s.\n']);
    
   
    %% save the error field: in the order: bias, rms and tot
    extot(isnan(extot))=0; eytot(isnan(eytot))=0;
    mask=(u~=0); extot=extot.*mask; eytot=eytot.*mask;
    filename=[fileroot zerostr(1:end-length(fieldstr)) fieldstr];
      
    % pixels
    Xw_px=Xu_px'-1; Yw_px=Yu_px'-1; uw_px=squeeze(displ_px(:,:,1))'; vw_px=squeeze(-displ_px(:,:,2))'; Nw=Npartu';
    exw_px=extot'; eyw_px=eytot';
    savematrix_px=[Xw_px(:) Yw_px(:) uw_px(:) vw_px(:) exw_px(:) eyw_px(:) Nw(:)];
    savematrix_px(isnan(savematrix_px))=0;
    savematrix_px(isinf(savematrix_px))=0;
   
    if strcmp(SaveFlag,'yes')       
        % pixels
        fid=fopen([fold_write filename app_write],'w');
        fprintf(fid,'%s\n',['TITLE = "' filename '"']);
        fprintf(fid,'%s\n','VARIABLES = "x", "y", "u", "v", "ex", "ey", "N"');
        fprintf(fid,'%s\n',['ZONE T="' filename '", I=' num2str(size(Xu_px,2)) ', J=' num2str(size(Xu_px,1))]);
        fprintf(fid,'%u %u %.6f %.6f %.6f %.6f %.1f \n',savematrix_px');
        fclose(fid);  
    end % SaveFlag
    
    tfield=toc;
    fprintf(['Time for the field: ' num2str(tfield-t1,'%4.1f') ' s.\n']);
    fprintf('\n')
end % loop on the displacement fields

tfinal=toc;
fprintf(['Total time: ' num2str(tfinal-t0,'%4.1f') ' s.\n'])

% to terminate the existing parallel session: 
% delete(gcp('nocreate'));

