function [etot ebias erms Npart C] = error_window4(displ,peaks,grdx,grdy,ws,nRmsLength,position_weight,ROI)
% function that computes the "bias" and rms error averaging the errors in an interrogation window
% peaks: 2D matrix of size (J,I): it is zero where no peaks are present, in
% the other points it expresses the peak displacement
% grdx: vector of the grid points in the x direction
% grdy: vector of the grid points in the y direction
% ws: window size

%% initialization
lenx=length(grdx); leny=length(grdy); [J,I]=size(peaks);
% if grdy(end)<grdy(1) grdy=flipud(grdy); end %to have both grdx and grdy in increasing order
r=(ws-1)/2; %radius of the window
if strcmp(position_weight,'tophat')
    coeff=1;
    gaussweight=ones(2*r+1);
elseif strcmp(position_weight,'gauss')
    coeff=1.75;%coeff=1.4;
    gaussweight=gaussian2De_2(round(2*coeff*r+1),round(2*coeff*r+1));
else
    error('"position_weight" has to be "tophat" or "gauss"');
end

ebias=zeros(leny,lenx); erms=zeros(leny,lenx);
Npart=zeros(leny,lenx);
etot=Npart;
% C=cell(leny,lenx);
C=0;

%% main loop
for jgrid=1:leny
    j=grdy(jgrid);
    jmin=max(1,j-coeff*r); jmax=min(J,j+coeff*r);
    for igrid=1:lenx
        i=grdx(igrid);
        
        if (i >= ROI(1) && i <= ROI(2) && j>=ROI(3) && j<=ROI(4))
            imin=max(1,i-coeff*r); imax=min(I,i+coeff*r);
            
            displwin=displ(jmin:jmax,imin:imax); %displacement in the window
            gaussloc=gaussweight(1+coeff*r-(j-jmin):1+coeff*r+jmax-j,1+coeff*r-(i-imin):1+coeff*r+(imax-i));
            peakswin=peaks(jmin:jmax,imin:imax); %peaks in the window
            peakswin=peakswin.*gaussloc;
            peaksold=peakswin;
            displold=displwin;
            displwin=displold(displold~=0);
            peakswin=peakswin(displold~=0);
            
            % outliers removal
            outliers=OutliersPercentile(displwin);
            displwin(outliers==1)=[];
            peakswin(outliers==1)=[];
            
            % computation
            ebiasloc=sum(displwin.*peakswin)./sum(peakswin);
            ermsloc=sqrt(sum(peakswin.*(displwin-ebiasloc).^2)/sum(peakswin));
            ebias(jgrid,igrid)=ebiasloc;
            Nweight=(peaksold>0).*gaussloc; Nweight=sum(Nweight(:));
            if Nweight<1 Nweight=1; end
            erms(jgrid,igrid)=ermsloc;
            etot(jgrid,igrid)=sqrt(ermsloc.^2/Nweight+ebiasloc.^2);
            Npart(jgrid,igrid)=Nweight;%N
        end % if ROI
    end
end


ebias(isnan(ebias))=0;
erms(isnan(erms))=0;
etot(isnan(etot))=0;
end % end function

